@@ -96,7 +96,7 @@ sub signal
   # if the signal is DIE, DESTROY yourself
   if ($sig == SIG_DIE)
     {
-    $self->DESTROY();
+    $self->kill();
     return;
     }
   # if the signal is ACTIVATE or DEACTIVATE, (in)activate yourself
